﻿using System.Reflection;
using HeuristicLab.Collections;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;
using System;
using System.Linq;

namespace HeuristicLab.BioBoost.Operators.Mutation {
  [StorableClass]
  public class MultiCrossover : StochasticMultiBranch<ICrossover>, ICrossover, IStochasticOperator {

    public override bool CanChangeName {
      get { return false; }
    }

    protected override bool CreateChildOperation {
      get { return true; }
    }

    #region Construction & Cloning
    [StorableConstructor]
    protected MultiCrossover(bool isDeserializing) : base(isDeserializing) {}
    protected MultiCrossover(MultiCrossover orig, Cloner cloner) : base(orig, cloner) {}
    public MultiCrossover() {
      foreach (var t in ApplicationManager.Manager.GetTypes(typeof(ICrossover), Assembly.GetExecutingAssembly())) {
        if (t == this.GetType()) continue; // avoid infinite recursion
        ICrossover instance = null;
        try {
          instance = Activator.CreateInstance(t) as ICrossover;
        } catch {}
        if (instance != null) Operators.Add(instance);
      }
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MultiCrossover(this, cloner);
    }
    #endregion

    public override IOperation InstrumentedApply() {
        if (Operators.Count == 0) throw new InvalidOperationException(Name + ": Please add at least one crossover to choose from.");
        return base.InstrumentedApply();
    }
  }
}
